from pydantic import BaseModel


class Config(BaseModel):
    """Plugin Config Here"""
    boardgamehelper_database_url: str = "sqlite:///./data/BoardGameHelper/database.db"
    boardgamehelper_json_path: str = "./data/BoardGameHelper/json/"



import json
from datetime import datetime, timedelta
from pathlib import Path
from typing import Any, Union
from zoneinfo import ZoneInfo

from sqlalchemy import URL, create_engine
from sqlalchemy.dialects.sqlite import insert
from sqlalchemy.orm import Session, sessionmaker

from .models import Base

BJ_TZ = ZoneInfo("Asia/Shanghai")
"""
说真的，如果你能帮我重构一下这个部分就太好了
我真不擅长对付ORM

TODO: 迁移至nonebot-plugin-orm
"""

class DataBaseManager:
    """
    该类负责将数据写入数据库或将实例从数据库里读出

    Args:
        db_path("Path"): 指向数据库的path对象
    """
    def __init__(self, db_path: "Path") -> None:
        url = URL.create(drivername="sqlite", database=str(db_path))
        self.engine = create_engine(url, echo=False)
        Base.metadata.create_all(self.engine)
        self.SessionLocal = sessionmaker(bind=self.engine)

    def get_session(self) -> Session:
        return self.SessionLocal()

    def insert(self, model: type[Base], data: dict) -> None:
        """
        在数据库增加条目
        说真的，你有多想不开要用这些增改删查方法，直接用upsert好了

        Args:
            model(type[Base]): ORM模型，接受所有继承于models里的Base基类的类
            data(dict): 要写入的数据
        """
        with self.get_session() as session:
            obj = model(**data)
            session.add(obj)
            session.commit()

    def update(self, model: type[Base], filters: dict, values: dict) -> None:
        """
        在数据库增加条目
        说真的，你有多想不开要用这些增改删查方法，直接用upsert好了

        Args:
            model(type[Base]): ORM模型，接受所有继承于models里的Base基类的类
            filters(dict): WHERE 条件
            values(dict): 要更新的字段
        """
        with self.get_session() as session:
            session.query(model).filter_by(**filters).update(values)
            session.commit()

    def delete(self, model: type[Base], filters: dict) -> None:
        """
        在数据库删除条目
        说真的，你有多想不开要用这些增改删查方法，直接用upsert好了
        不对，这个不能用upsert

        Args:
            model(type[Base]): ORM模型，接受所有继承于models里的Base基类的类
            filters(dict): WHERE 条件
        """
        with self.get_session() as session:
            session.query(model).filter_by(**filters).delete()
            session.commit()

    def select(
            self,
            model: type[Base],
            filters: dict,
            *,first: bool=False
            )  -> Union[Base, list[Base], None]:
        """
        在数据库查询条目
        说真的，你有多想不开要用这些增改删查方法，直接用upsert好了
        不对，这个不能用upsert

        Args:
            model(type[Base]): ORM模型，接受所有继承于models里的Base基类的类
            filters(dict): WHERE 条件
            first(boolean): 是否只返回第一条 default=False
        """
        with self.get_session() as session:
            query = session.query(model)
            if filters:
                query = query.filter_by(**filters)
            return query.first() if first else query.all()

    def upsert(self, model: type[Base], data: dict) -> None:
        """
        在数据库插入或者更新条目
        SQLite 专用 UPSERT
        SQLite3.24最伟大的发明

        Args:
            model(type[Base]): ORM模型，接受所有继承于models里的Base基类的类
            data(dict): 要写入的数据
        """
        with self.get_session() as session:
            stmt = insert(model).values(**data)
            primary_keys = [c.name for c in model.__table__.primary_key]
            update_data = {k: v for k, v in data.items() if k not in primary_keys}
            upsert_stmt = stmt.on_conflict_do_update(
                index_elements=primary_keys,
                set_=update_data
            )
            session.execute(upsert_stmt)
            session.commit()

    def cleanup_expired(
        self,
        model: type[Base],
        time_field: Any,
        minutes: int = 1
    ) -> int:
        """
        清理数据库中指定时间字段超时的条目

        Args:
            model(type[Base]): ORM 模型
            time_field(Any): 数据库模型中的datetime字段，如RecruitmentPosts.end_time
            minutes(int): 超时时间，单位分钟，默认1分钟

        Returns:
            int: 被删除的条目数量
        """
        now = datetime.now(BJ_TZ)  # 如果字段带时区可改成 datetime.now(BJ_TZ)
        threshold = now - timedelta(minutes=minutes)
        threshold = threshold.replace(microsecond=0)
        with self.get_session() as session:
            query = session.query(model).filter(time_field < threshold)
            count = query.count()
            query.delete(synchronize_session=False)
            session.commit()
            return count

class JsonIO:
    """
    该类负责使用 pathlib 读写 JSON 文件为字典

    Attr:
        path ("Path"): 指向JSON文件的Path对象
    """
    def __init__(self, path: Path) -> None:
        self.path = path

    def load(self) -> dict:
        """
        从 JSON 文件加载数据

        Returns:
            dict: JSON 内容，如果文件不存在返回空字典
        """
        if not self.path.exists():
            return {}
        return json.loads(self.path.read_text(encoding="utf-8"))

    def save(self, data: dict) -> None:
        """
        将字典保存到 JSON 文件

        Args:
            data (dict): 要保存的数据
        """
        self.path.parent.mkdir(parents=True, exist_ok=True)
        self.path.write_text(json.dumps(data, ensure_ascii=False, indent=4), encoding="utf-8")


from datetime import datetime

from sqlalchemy import DateTime, String
from sqlalchemy.orm import DeclarativeBase, Mapped, mapped_column


class Base(DeclarativeBase):
    pass

class PostsModel(Base):
    __tablename__ = "recruitment_posts"

    recruitment_code: Mapped[str] = mapped_column(String, primary_key=True)
    publisher_user_id: Mapped[str] = mapped_column(String)
    publisher_name: Mapped[str] = mapped_column(String)
    content: Mapped[str] = mapped_column(String)
    end_time: Mapped[datetime] = mapped_column(DateTime)

class BroadcastModel(Base):
    __tablename__ = "broadcast_groups"

    group_id: Mapped[str] = mapped_column(String, primary_key=True)
    group_name: Mapped[str] = mapped_column(String)

class PictpackMode(Base):
    __tablename__ = "picture_packs"

    pack_code: Mapped[str] = mapped_column(String, primary_key = True)
    pack_name: Mapped[str] = mapped_column(String)
    copyright_info: Mapped[str] = mapped_column(String)
    pack_like: Mapped[str] = mapped_column(String)
    pack_unlike: Mapped[str] = mapped_column(String)
    pack_download_link: Mapped[str] = mapped_column(String)
    pack_fast_download: Mapped[str] = mapped_column(String)
    pack_info: Mapped[str] = mapped_column(String)



import re
from datetime import datetime
from enum import Enum, auto
from typing import Optional
from zoneinfo import ZoneInfo


class DateCheckResult(Enum):
    OK = auto()
    INVALID_FORMAT = "输入格式不合规"
    INVALID_SEQUENCE = "早于当前时间"
    INVALID_DATETIME = "不存在的日期或时间"
    ONLY_DATE = "只提供日期，未提供时间"

class ContentCheckResult(Enum):
    OK = auto()
    MISSING_BOARDGAME = "缺少[桌游名]"
    MISSING_PLAYERS = "缺少[人数]"
    MISSING_ROOM = "缺少[房间名/房间密码]"
    MISSING_TUTORIAL = "缺少[是否含教学]"
    MISSING_VOICE = "缺少[语音房间链接]"

# 北京时间 UTC+8
BJ_TZ = ZoneInfo("Asia/Shanghai")



def _need_insert_newlines(text: str) -> bool:
    max_square_brackets = 2
    """
    判断是否存在“同一行多个字段头”的情况
    """
    return any(line.count("[") >= max_square_brackets for line in text.splitlines())

def validate_datetime(
        input_str: str
        ) -> tuple[list["DateCheckResult"], Optional[datetime]]:
    """
    时间校验器

    Args:
        input_str (str): 用户输入的日期时间字符串

    Returns:
        tuple:
            - list[DateCheckResult]: 不符合的检查项，如果符合返回空列表
            - datetime | None: 合规的时间，或 None
    """
    input_str = input_str.strip().replace("：", ":")
    now = datetime.now(BJ_TZ)
    dt: Optional[datetime] = None
    errors: list["DateCheckResult"] = []

    # 尝试仅时间 → 补今天日期
    try:
        tmp_time = datetime.strptime(input_str, "%H:%M").time()  # noqa: DTZ007 可以肯定用户输入的均为北京时间
        if tmp_time.hour >= 24 or tmp_time.minute >= 60:  # noqa: PLR2004 时间校验没必要写变量了吧...
            errors.append(DateCheckResult.INVALID_DATETIME)
        else:
            today = now.date()
            tmp_dt = datetime.combine(today, tmp_time, tzinfo=BJ_TZ)
            if tmp_dt < now:
                errors.append(DateCheckResult.INVALID_SEQUENCE)
            else:
                dt = tmp_dt
                return [DateCheckResult.OK], dt
    except ValueError:
        pass

    # 尝试仅日期
    if dt is None:
        try:
            # .date()如果匹配了，就代表输入仅有年月日，所以到这里应该出现ValueError
            datetime.strptime(input_str, "%Y-%m-%d").date()  # noqa: DTZ007 可以肯定用户输入的均为北京时间，时区赋值已在下一行实现
            errors.append(DateCheckResult.ONLY_DATE)
        except ValueError:
            pass

    # 尝试完整日期时间
    if dt is None:
        try:
            tmp = datetime.strptime(input_str, "%Y-%m-%d %H:%M")  # noqa: DTZ007 可以肯定用户输入的均为北京时间，时区赋值已在下一行实现
            tmp = tmp.astimezone(BJ_TZ)
            if tmp.hour >= 24 or tmp.minute >= 60:  # noqa: PLR2004 时间校验没必要写变量了吧...
                errors.append(DateCheckResult.INVALID_DATETIME)
            elif tmp < now:
                errors.append(DateCheckResult.INVALID_SEQUENCE)
            else:
                dt = tmp
                return [DateCheckResult.OK], dt
        except ValueError:
            errors.append(DateCheckResult.INVALID_FORMAT)


    return errors, dt

def _pre_process(input_str: str) -> str:
    input_str = input_str.replace("【", "[").replace("】", "]")
    patterns = [
        re.escape("查找图包可以输入（图包查询：桌游名）"),
        re.escape("接下来请输入发车信息，例："),
        re.escape("请在一分钟内内完成输入，超时后您需要重新发车"),
        re.escape("请输入你的发车信息,如：")
    ]
    combined_pattern = "|".join(patterns)
    before_content = r"^[^\[]+"
    input_str = re.sub(combined_pattern, "", input_str).strip()
    input_str = re.sub(before_content, "", input_str).strip()
    return input_str  # noqa: RET504 shut fuck up 如果之后有其他正则匹配需求怎么办

def validate_content(input_str: str) -> tuple[list[ContentCheckResult], str]:
    """
    校验招募文本是否包含必要字段
    当有任意一项校验未通过时，返回空字符串

    returns:
        tuple(list[enum, str])
    """
    normalized = _pre_process(input_str)
    # 定义检测规则
    checks = {
        ContentCheckResult.MISSING_BOARDGAME: r"\[桌游名\](.*?)\n"
    }

    # 自动补换行
    if _need_insert_newlines(normalized):
        normalized = re.sub(r"(?<!^)\[", "\n[", normalized)

    missing: list[ContentCheckResult] = []

    for enum_key, pattern in checks.items():
        if not re.search(pattern, normalized):
            missing.append(enum_key)

    # 有缺失则不返回字符串，返回所有丢失值的枚举
    if missing:
        return (missing, "")

    # 全部存在后返回枚举值OK和格式化好的字符串
    return ([ContentCheckResult.OK], normalized)



from datetime import datetime
from typing import TYPE_CHECKING

import shortuuid
from nonebot_plugin_alconna import Text, UniMessage

if TYPE_CHECKING:
    from .models import PostsModel


class Post:
    """
    一个招募标准类

    attr:
        recruitment_code: 招募信息的标识id
        publisher_user_id: 发起招募信息者的id
        publisher_name: 发起招募信息者的昵称
        content: 招募信息内容
        end_time: 招募结束时间
    """
    def __init__(
            self, user_id: str,
            user_name: str,
            content: str,
            end_time: datetime
            ) -> None:
        self.recruitment_code: str = shortuuid.ShortUUID().random(4)
        self.publisher_user_id = user_id
        self.publisher_name = user_name
        self.content = content
        self.end_time = end_time

    def to_dict(self) -> dict:
        """
        该方法用于将招募类转换为字典
        """
        return{
            "recruitment_code": self.recruitment_code,
            "publisher_user_id": self.publisher_user_id,
            "publisher_name": self.publisher_name,
            "content": self.content,
            "end_time": self.end_time
        }
    @classmethod
    def from_orm_class(cls, data: "PostsModel") -> "Post":
        """
        该方法用于从orm模板类恢复为Post实例
        """
        post = cls(
            user_id = data.publisher_user_id,
            user_name = data.publisher_name,
            content = data.content,
            end_time = data.end_time
        )
        post.recruitment_code = data.recruitment_code
        return post

    def to_unimessage(self) -> UniMessage:
        """
        该方法用于将Post实例转换为UniMessage类
        """
        return UniMessage([
            Text(f"[车车ID]{self.recruitment_code}"),
            Text("\n"),
            Text(f"[发车人]{self.publisher_name}({self.publisher_user_id})"),
            Text("\n"),
            Text(self.content),
            Text("\n"),
            Text(f"[结束时间]{self.end_time.strftime('%Y-%m-%d %H:%M')}")
        ])




from typing import TYPE_CHECKING

from .data_manager import JsonIO

if TYPE_CHECKING:
    from pathlib import Path


def reply_generator(path: "Path") -> None:
    json_file = JsonIO(path)
    reply = {
        "already_publish": "您已经发了一辆车",
        "publish_tip": "接下来请输入发车信息，例：\n[桌游名] 璀璨宝石\n[人数] 1=3\n[房间名/房间密码] room/password\n[是否含教学] 是\n[语音房间链接] https://mornin.fm\n请在一分钟内内完成输入，超时后您需要重新发车",
        "time_tip": "接下来请输入自动封车的时间，如\n18:00\n2025-07-21 19:19\n当您仅输入时间时，将会默认日期为当天",
        "publish_missing_sth": "您的发车信息缺少条目：",
        "please_check": "请检查后重新发车",
        "date_missing_sth": "您的结束时间有以下问题：",
        "published": "您的发车信息已经存入数据库并广播至其他群",
        "no_recruitments": "现在还没有车",
        "recruitments_now": "以下为一天内时间的车",
        "recruitments_after": "以下为一天后时间的车",
        "close_recruitment_success": "封车完成",
        "close_recruitment_failed": "封车失败：\n车库里没有您发的车呢",
        "ask_for_uuid": "请输入想要封掉的车的车车ID",
        "delete_recruitment_success": "强制封车成功",
        "delete_recruitment_failed": "强制封车失败，车库里没有该编号的车",
        "broadcast_exists": "本群已经开启过了全群广播",
        "open_broadcast_success": "本群开启了全群广播",
        "broadcast_not_exists": "本群尚未开启全群广播，无法关闭",
        "close_broadcast_success": "本群关闭了全群广播"
    }
    json_file.save(reply)



from datetime import datetime, timedelta
from typing import cast
from zoneinfo import ZoneInfo

from nonebot import get_plugin_config, logger, on_command, require


require("nonebot_plugin_alconna")
require("nonebot_plugin_apscheduler")
require("nonebot_plugin_localstore")

import nonebot_plugin_localstore as storage
from nonebot.adapters.onebot.v11 import (
    GROUP_ADMIN,
    GROUP_OWNER,
    Bot,
    GroupMessageEvent,
    Message,
    MessageEvent,
    exception,
)
from nonebot.params import ArgPlainText, CommandArg
from nonebot.permission import SUPERUSER
from nonebot.plugin import PluginMetadata
from nonebot.typing import T_State
from nonebot_plugin_alconna import CustomNode, SupportScope, Target, UniMessage
from nonebot_plugin_apscheduler import scheduler

from .config import Config
from .data_manager import DataBaseManager, JsonIO
from .models import BroadcastModel, PostsModel
from .post_func import Post
from .utils import reply_generator
from .validator import (
    ContentCheckResult,
    DateCheckResult,
    validate_content,
    validate_datetime,
)

__plugin_meta__ = PluginMetadata(
    name="nonebot_plugin_boardgamehelper",
    description="一个 NoneBot2 桌游约车助手插件，提供桌游群招募、发车、封车等功能。",
    usage="",
    type="application",
    homepage="https://github.com/SaltedFish0208/nonebot-plugin-boardgamehelper",
    config=Config,
    supported_adapters={"~onebot.v11"}
)


config = get_plugin_config(Config)
BJ_TZ = ZoneInfo("Asia/Shanghai")


db_path = storage.get_plugin_data_file("database.db")
reply_path = storage.get_plugin_config_file("reply.json")

db_path.touch(exist_ok=True)
if not reply_path.exists():
    reply_generator(reply_path)

db = DataBaseManager(db_path)
reply = JsonIO(reply_path).load()

publish_recruitment = on_command("发车", aliases={"桌游发车", "开车"})
@publish_recruitment.handle()
async def _(event: MessageEvent, state: T_State):
    if db.select(
        PostsModel,
        {
            "publisher_user_id": event.user_id
        },
        first=True
        ) is not None:
        await UniMessage.text(reply["already_publish"]).finish()
    await UniMessage.text(reply["publish_tip"]).send()
    state["publisher_user_id"] = event.user_id
    state["publisher_name"] = event.sender.nickname

@publish_recruitment.got("recruitment_content")
async def _(state: T_State, message: str = ArgPlainText("recruitment_content")):
    result = validate_content(message)
    if ContentCheckResult.OK not in result[0]:
        error_list = [r.value for r in result[0]]
        error_list = cast("str", error_list)
        missing = "\n".join(error_list)
        await UniMessage.text(reply["publish_missing_sth"]).text("\n").text(missing).text("\n").text(reply["please_check"]).finish()  # noqa: E501
    state["recruitment_content"] = result[1]
    await UniMessage.text(reply["time_tip"]).send()

@publish_recruitment.got("end_time")
async def _(state:T_State, message: str = ArgPlainText("end_time")):
    result = validate_datetime(message)
    if DateCheckResult.OK not in result[0]:
        error_list = [r.value for r in result[0]]
        error_list = cast("str", error_list)
        missing = "\n".join(error_list)
        await UniMessage.text(reply["date_missing_sth"]).text("\n").text(missing).text("\n").text(reply["please_check"]).finish()  # noqa: E501
    time = cast("datetime", result[1])
    post = Post(
        user_id=state["publisher_user_id"],
        user_name=state["publisher_name"],
        content=state["recruitment_content"],
        end_time=time
    )
    broadcast_groups = db.select(BroadcastModel, {}, first=False)
    broadcast_groups = cast("list[BroadcastModel]", broadcast_groups)
    ids = [group.group_id for group in broadcast_groups]
    if ids:
        for group_id in ids:
            target = Target(group_id, scope=SupportScope.qq_client)
            try:
                await post.to_unimessage().send(target=target)
            except exception.ActionFailed:
                db.delete(BroadcastModel, {"group_id": group_id})
    packaged = post.to_dict()
    db.upsert(PostsModel, packaged)
    await UniMessage.text(reply["published"]).finish()

query_recruitment = on_command("查车", aliases={"桌游查车"})
@query_recruitment.handle()
async def _(bot: Bot):
    all_recruitments = db.select(PostsModel, {}, first=False)
    all_recruitments = cast("list[PostsModel]", all_recruitments)
    recruitments_now = []
    recruitments_after = []
    now = datetime.now(BJ_TZ)
    if not all_recruitments:
        await UniMessage.text(reply["no_recruitments"]).finish()
    for recruitment in all_recruitments:
        post = Post.from_orm_class(recruitment)
        recruitment.end_time = recruitment.end_time.replace(tzinfo=BJ_TZ)
        if abs(now - recruitment.end_time) <= timedelta(days=1):
            recruitments_now.append(post.to_unimessage())
        else:
            recruitments_after.append(post.to_unimessage())
    recruitments_now.insert(0, UniMessage.text(reply["recruitments_now"]))
    recruitments_after.insert(0, UniMessage.text(reply["recruitments_after"]))
    seq = recruitments_now + recruitments_after
    await UniMessage.reference(*[
            CustomNode(uid=bot.self_id, name="Amadeus", content=msg)
            for msg in seq
        ]
    ).finish()

close_recruitment = on_command("封车", aliases={"桌游封车"})
@close_recruitment.handle()
async def _(event: MessageEvent):
    target = db.select(PostsModel, {"publisher_user_id": event.user_id}, first=True)
    if target is None:
        await UniMessage.text(reply["close_recruitment_failed"]).finish()
    db.delete(PostsModel, {"publisher_user_id": event.user_id})
    await UniMessage.text(reply["close_recruitment_success"]).finish()

delete_recruitment = on_command("强制封车", permission=SUPERUSER)
@delete_recruitment.handle()
async def _(msg: Message = CommandArg()):
    msg_str = msg.extract_plain_text().strip()
    if not msg:
        await UniMessage.text(reply["ask_for_uuid"]).send()
    elif db.select(PostsModel, {"recruitment_code": msg_str}, first=True) is None:
        await UniMessage.text(reply["delete_recruitment_failed"]).finish()
    else:
        db.delete(PostsModel, {"recruitment_code": msg_str})
        await UniMessage.text(reply["delete_recruitment_success"]).finish()

@delete_recruitment.got("recruitment_code")
async def _(msg: Message = CommandArg()):
    msg_str = msg.extract_plain_text().strip()
    if db.select(PostsModel, {"recruitment_code": msg_str}, first=True) is None:
        await UniMessage.text(reply["delete_recruitment_failed"]).finish()
    else:
        db.delete(PostsModel, {"recruitment_code": msg_str})
        await UniMessage.text(reply["delete_recruitment_success"]).finish()

open_broadcast = on_command(
    "开启全群广播",
    permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER
    )
@open_broadcast.handle()
async def _(bot: Bot, event: GroupMessageEvent):
    if db.select(BroadcastModel, {"group_id": event.group_id}, first=True) is not None:
        await UniMessage.text(reply["broadcast_exists"]).finish()
    group_info = await bot.call_api("get_group_info", group_id=event.group_id)
    db.upsert(
        BroadcastModel,
        {
            "group_id": event.group_id,
            "group_name": group_info["group_name"]
        })
    await UniMessage.text(reply["open_broadcast_success"]).finish()

close_broadcast = on_command(
    "关闭全群广播",
    permission=SUPERUSER|GROUP_ADMIN|GROUP_OWNER
    )
@close_broadcast.handle()
async def _(event: GroupMessageEvent):
    if db.select(BroadcastModel, {"group_id": event.group_id}, first=True) is None:
        await UniMessage.text(reply["broadcast_not_exists"]).finish()
    db.delete(BroadcastModel, {"group_id": event.group_id})
    await UniMessage.text(reply["close_broadcast_success"]).finish()

reload_reply = on_command("重载回复")
@reload_reply.handle()
async def _():
    global reply  # noqa: PLW0603 目前重载的只有这一个json，之后再修吧
    reply = JsonIO(reply_path).load()
    await UniMessage.text("success").finish()


@scheduler.scheduled_job("interval", minutes=1)
async def _():
    count = db.cleanup_expired(PostsModel, PostsModel.end_time, 0)
    logger.info(f"清理了 {count} 条过时信息")
